@@ -1,18 +1,30 @@
 """ Model for Sun features """
-from pydantic import Field, validator
-import numpy as np
-import xarray as xr
-
-from nowcasting_dataset.data_sources.datasource_output import DataSourceOutput
-from nowcasting_dataset.consts import Array, SUN_AZIMUTH_ANGLE, SUN_ELEVATION_ANGLE
-from nowcasting_dataset.utils import coord_to_range
-from nowcasting_dataset.time import make_random_time_vectors
 import logging
 
+import numpy as np
+from pydantic import Field, validator
+
+from nowcasting_dataset.consts import Array, SUN_AZIMUTH_ANGLE, SUN_ELEVATION_ANGLE
+from nowcasting_dataset.data_sources.datasource_output import (
+    DataSourceOutputML,
+    DataSourceOutput,
+)
+from nowcasting_dataset.time import make_random_time_vectors
+
 logger = logging.getLogger(__name__)
 
 
 class Sun(DataSourceOutput):
+    """ Class to store Sun data as a xr.Dataset with some validation """
+
+    # Use to store xr.Dataset data
+    __slots__ = ()
+    _expected_dimensions = ("time",)
+
+    # todo add validation here - https://github.com/openclimatefix/nowcasting_dataset/issues/233
+
+
+class SunML(DataSourceOutputML):
     """ Model for Sun features """
 
     sun_azimuth_angle: Array = Field(
@@ -55,7 +67,7 @@ class Sun(DataSourceOutput):
                 batch_size=batch_size, seq_length_5_minutes=seq_length_5, seq_length_30_minutes=0
             )
 
-        return Sun(
+        return SunML(
             batch_size=batch_size,
             sun_azimuth_angle=np.random.randn(
                 batch_size,
@@ -72,40 +84,11 @@ class Sun(DataSourceOutput):
         """ Get the datetime index of this data """
         return self.sun_datetime_index
 
-    def to_xr_dataset(self, i):
-        """ Make a xr dataset """
-        logger.debug(f"Making xr dataset for batch {i}")
-        individual_datasets = []
-        for name in [SUN_AZIMUTH_ANGLE, SUN_ELEVATION_ANGLE]:
-
-            var = self.__getattribute__(name)
-
-            data = xr.DataArray(
-                var,
-                dims=["time"],
-                coords={"time": self.sun_datetime_index},
-                name=name,
-            )
-
-            ds = data.to_dataset()
-            ds = coord_to_range(ds, "time", prefix=None)
-            individual_datasets.append(ds)
-
-        data = xr.DataArray(
-            self.sun_datetime_index,
-            dims=["time"],
-            coords=[np.arange(len(self.sun_datetime_index))],
-        )
-        ds = data.to_dataset(name="sun_datetime_index")
-        individual_datasets.append(ds)
-
-        return xr.merge(individual_datasets)
-
     @staticmethod
     def from_xr_dataset(xr_dataset):
         """ Change xr dataset to model. If data does not exist, then return None """
         if SUN_AZIMUTH_ANGLE in xr_dataset.keys():
-            return Sun(
+            return SunML(
                 batch_size=xr_dataset[SUN_AZIMUTH_ANGLE].shape[0],
                 sun_azimuth_angle=xr_dataset[SUN_AZIMUTH_ANGLE],
                 sun_elevation_angle=xr_dataset[SUN_ELEVATION_ANGLE],
